#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import zip
import numpy.testing as npt
import os
import pytest
import pandas as pd

from cornaconvertor import script_convert as sc
from pandas.util.testing import assert_frame_equal
from cornaconvertor import helpers as hlp

dir_path = os.path.dirname(os.path.realpath(__file__))


def test_convert_file_LCMS():
    expected_df = pd.DataFrame({'Label': ['H2-label-1', 'H2-label-3'],
                    'Name': ['betaine', 'betaine'],
                    'SAMPLE_#SPSFJ7H_2_1': [6347.217, 0.000],
                    'SAMPLE_#SPSFJ7H_2_2': [ 0.000, 6438.023],
                    'Formula': ['C5H11NO2', 'C5H11NO2']})

    inputfile = os.path.join(dir_path,'lcms_test_data/test_data/LCMS_convert_test_file.csv')
    output_df = sc.convert_file_LCMS(inputfile)
    npt.assert_array_equal(expected_df, output_df.sort_index(axis=0))


def test_convert_file_MSMS():
    expected_df = pd.DataFrame({'Component Name': ['Succinate 121/76', 'Succinate 121/76'],
                                'Original Filename': ['TA_13C-glucose with varying [glc]_2Sept13-G. [13C-glc] '
                                                      'G9 240min (sample=344) | Sample Number = 344',
                                                      'TA_13C-glucose with varying [glc]_2Sept13-G. [13C-glc] '
                                                      'G9 240min (sample=336) | Sample Number = 336'],
                                'Area': [45700, 42800],
                                'Method': ['cit', 'cit'],
                                'Mass Info': ['121.0 / 76.0', '121.0 / 76.0']})
    inputfile = os.path.join(dir_path,'msms_test_input/convert_test_file.csv')
    output_df = sc.convert_file_MSMS(inputfile)
    npt.assert_array_equal(expected_df, output_df[0].sort_index(axis=1))


def test_convert_sample_qc():
    # TODO: This test case need to be corrected as per the index of the output_df
    expected_df = pd.DataFrame({'sample_id':['091215_5i_0','091215_5M_0'],
                                'C12 PARENTacetyl-CoA': [2621.898, 4674.641],
                                'C13-label-1acetyl-CoA': [984.978, 1048.766],
                                })
    expected_df = expected_df.set_index('sample_id')
    inputfile = os.path.join(dir_path, 'lcms_test_data/test_data/test_maven_to_sample.csv')
    output_df = sc.convert_maven_to_sample_qc(inputfile)
    npt.assert_array_equal(expected_df, output_df.sort_index(axis=1))

testdata = [(os.path.join(dir_path,
                          'lcms_test_data/test_data/test_maven_to_pvd.csv'),

             pd.DataFrame({'compoundId': ['neg_muscle_9', 'neg_muscle_10'],
                           'HMDB00243': [2470000000, 512000000],
                           'HMDB11111': [2470000000, 512000000]})),
            (os.path.join(dir_path,
                          'lcms_test_data/test_data/test_maven_to_pvd_label'
                          '.csv'),
             pd.DataFrame({'compoundId': ['neg_muscle_9', 'neg_muscle_10'],
                           'HMDB00243': [2470000000, 512000000],
                           'HMDB11111': [2470000000, 512000000]}) )]

@pytest.mark.parametrize("path,expected", testdata)
def test_convert_maven_to_unlabelled_pvd(path, expected):
    """ Test to check expected output after converting to maven df"""
    expected.set_index('compoundId', inplace=True)
    output_df = sc.convert_maven_to_unlabelled_pvd(path)
    npt.assert_array_equal(expected, output_df)


@pytest.mark.parametrize("path, expected", [("", 'The file does not exist')])
def test_convert_maven_to_unlabelled_pvd_exc(path, expected):
    """
    Test to check error if input path does not exist
    """
    with pytest.raises(IOError):
        sc.convert_maven_to_unlabelled_pvd(path)

def test_convert_isocorrect_to_quant_lcms():
    inputfile = os.path.join(dir_path,'lcms_test_data/Polly/raw_intensity_file_na_corrected.csv')
    path = os.path.join(dir_path,'lcms_test_data/Polly/converted_quant.csv')
    output_df = sc.convert_isocorrect_to_quant_lcms(inputfile)
    expected_df = hlp.read_file(path)
    # npt.assert_array_equal(expected_df, output_df.sort_index(axis=1))
    assert all([a == b for a, b in zip(output_df, expected_df)])

def test_convert_isocorrect_to_quickcheck_lcms():
    inputfile = os.path.join(dir_path,'lcms_test_data/Polly/raw_intensity_file_na_corrected.csv')
    path = os.path.join(dir_path,'lcms_test_data/Polly/converted_naqc.csv')
    output_df = sc.convert_isocorrect_to_quickcheck_lcms(inputfile)
    expected_df = hlp.read_file(path)
    assert all([a == b for a, b in zip(output_df, expected_df)])

def test_convert_quantfit_to_quickcheck_lcms():
    inputfile = os.path.join(dir_path,'lcms_test_data/Polly/quant_raw_file.csv')
    path = os.path.join(dir_path,'lcms_test_data/Polly/converted_qfqc_1.csv')
    output_df = sc.convert_quantfit_to_quickcheck_lcms(inputfile)
    expected_df = hlp.read_file(path)
    assert all([a == b for a, b in zip(output_df, expected_df)])

def test_convert_quantfit_to_metscape_lcms():
    inputfile = os.path.join(dir_path,'lcms_test_data/Polly/quant_raw_file.csv')
    path = os.path.join(dir_path,'lcms_test_data/Polly/converted_metscape.csv')
    output_df = sc.convert_quantfit_to_quickcheck_lcms(inputfile)
    expected_df = hlp.read_file(path)
    assert all([a == b for a, b in zip(output_df, expected_df)])
